# -*- coding: utf-8 -*-

"""
@date: 2020/7/26 上午10:36
@file: misc.py
@author: zj
@description: 
"""

from .. import __version__
import os


def get_version():
    return __version__


def is_dir(src_dir):
    """
    Determine whether it is a directory
    """
    if not os.path.isdir(src_dir):
        raise ValueError('{} is not a directory'.format(src_dir))


def get_img_name(img_path):
    """
    Resolve the image name. like /path/to/lena.jpg get lena
    """
    return os.path.splitext(os.path.split(img_path)[1])[0]


def parse_img_path(img_path):
    """
    Analyze the image path, get the image folder, file name, image name and image suffix
    like /path/to/input_data/lena.jpg，then return
    ['input_data', 'lena.jpg', 'lena', '.jpg']
    """
    base_name = os.path.basename(img_path)
    dir_name = os.path.dirname(img_path)

    folder = os.path.basename(dir_name)
    file_name = base_name
    img_name, img_extension = os.path.splitext(base_name)

    return [folder, file_name, img_name, img_extension]


def get_classmap(anno_data, classmap):
    """
    Find classmap from middle format annotation data
    :param anno_list: [{'size':(width, height), 'objects':[{'name': '', 'bndbox':[xmin,ymin,xmax,ymax]}, ...]}, ...]
    :param classmap: Existing class name and number map
    :return: {'a': 0, 'b':1 ,...}
    """
    for _, anno_obj in anno_data.items():
        objects = anno_obj['objects']
        for obj in objects:
            name = obj['name']
            if name not in classmap.keys():
                classmap[name] = len(classmap.keys())
    return classmap


def check_image_label(img_path_list, anno_path_list):
    """
    Check whether the image file corresponds to the annotation file one by one
    """
    assert len(img_path_list) == len(anno_path_list), \
        'Number of image files: {} - Number of label files: {}'.format(len(img_path_list), len(anno_path_list))

    for img_path, anno_path in zip(img_path_list, anno_path_list):
        img_name = get_img_name(img_path)
        anno_name = get_img_name(anno_path)

        if img_name != anno_name:
            raise ValueError('{} and {} do not correspond'.format(img_path, anno_path))


def check_input_output_folder(dir: str, image_folder: str, label_folder: str, is_input=True):
    """
    Check the root directory as well as the image and label folders
    Check input directory
    1. Check whether the root path exists: if not, throw an exception
    2. Check whether the image folder exists: if not, throw an exception
    3. Check whether the label folder exists: if not, throw an exception
    Check output directory
    1. Check whether the root path exists: if not, create a new one
    2. Check whether the image folder exists: if so, throw an exception
    3. Check whether the label folder exists: if so, throw an exception
    :param dir: Root path
    :param image_folder: Image folder name
    :param label_folder: Label folder name
    :return: Return the image folder path and label folder path
    """
    assert isinstance(dir, str) and isinstance(image_folder, str) and isinstance(label_folder, str)

    img_dir = os.path.join(dir, image_folder)
    label_dir = os.path.join(dir, label_folder)

    if is_input:
        if not os.path.exists(dir) or not os.path.exists(img_dir) or not os.path.exists(label_dir):
            raise ValueError('Please check whether the input data is correct')
    else:

        if not os.path.exists(dir):
            os.mkdir(dir)

        if os.path.exists(img_dir):
            raise ValueError('{} already exists'.format(img_dir))
        os.mkdir(img_dir)

        if img_dir != label_dir:
            if os.path.exists(label_dir):
                raise ValueError('{} already exists'.format(label_dir))
            os.mkdir(label_dir)

    return img_dir, label_dir


def parse_classmap(classmap: dict, objects: list):
    """
    Parse the class name and add the corresponding number
    """
    assert isinstance(classmap, dict) and isinstance(objects, list)

    for obj in objects:
        name = obj['name']

        if name not in classmap.keys():
            classmap[name] = len(classmap.keys())



import logging
import os
import sys


def setup_logger(name, save_dir=None):
    logger = logging.getLogger(name)

    logger.propagate = False

    logger.setLevel(logging.DEBUG)
    stream_handler = logging.StreamHandler(stream=sys.stdout)
    formatter = logging.Formatter("%(asctime)s %(name)s %(levelname)s: %(message)s")
    stream_handler.setLevel(logging.DEBUG)
    stream_handler.setFormatter(formatter)
    logger.addHandler(stream_handler)

    if save_dir:
        fh = logging.FileHandler(os.path.join(save_dir, 'log.txt'), mode='a')
        fh.setLevel(logging.DEBUG)
        fh.setFormatter(formatter)
        logger.addHandler(fh)
    return logger



# -*- coding: utf-8 -*-

"""
@date: 2020/7/17 下午7:40
@file: visdrone_anno.py
@author: zj
@description: 
"""

import os
import glob
import numpy as np
import cv2

from . import registry
from ..anno.base_anno import BaseAnno
from ..util.misc import check_image_label, check_input_output_folder
from ..util.logger import setup_logger


@registry.ANNOS.register('visdrone')
class VisDroneAnno(BaseAnno):
    """
    [VisDrone](http://aiskyeye.com/) data set for UAV target detection training
    Download address: [VisDrone/VisDrone-Dataset](https://github.com/VisDrone/VisDrone-Dataset)
    Data storage format:
    root/
        images/
        annotations/
    The annotation file (. Txt) corresponds to the image one by one, and each line represents a annotation object, with a total of 8 fields
    refer to [Object Detection in Images](http://aiskyeye.com/evaluate/results-format/). The format is as follows
    <bbox_left>,<bbox_top>,<bbox_width>,<bbox_height>,<score>,<object_category>,<truncation>,<occlusion>
    Ignore in category ignored-regions/others
    """

    classmap = {'ignored-regions': 0, 'pedestrian': 1, 'people': 2, 'bicycle': 3, 'car': 4, 'van': 5, 'truck': 6,
                'tricycle': 7, 'awning-tricycle': 8, 'bus': 9, 'motor': 10, 'others': 11}

    def __init__(self, cfg):
        self.name = cfg.VISDRONE.NAME
        self.img_extension = cfg.VISDRONE.IMG_EXTENSION
        self.anno_extension = cfg.VISDRONE.ANNO_EXTENSION

        if cfg.ANNO.PARSER == self.name:
            self.src_dir = cfg.INPUT.DIR
            self.image_folder = cfg.INPUT.IMAGE_FOLDER
            self.label_folder = cfg.INPUT.LABEL_FOLDER
        if cfg.ANNO.CREATOR == self.name:
            self.dst_dir = cfg.OUTPUT.DIR
            self.image_folder = cfg.OUTPUT.IMAGE_FOLDER
            self.label_folder = cfg.OUTPUT.LABEL_FOLDER

        self.verbose = cfg.ANNO.VERBOSE

        self.logger = setup_logger(__name__)

    def get_class_name(self, value):
        return [k for k, v in self.classmap.items() if v == value][0]

    def create_anno(self, anno_line) -> dict:
        # ignore ignored-regions/others
        xmin, ymin, width, height, _, cate = anno_line[:6].astype(np.int)
        xmax = xmin + width
        ymax = ymin + height
        if cate == 0 or cate == 11:
            return dict()
        name = self.get_class_name(cate)
        return {'name': name, 'bndbox': [xmin, ymin, xmax, ymax]}

    def parse_anno(self, img_path, anno_path) -> dict:
        """
        Parse vistrone image and annotation file
        :returns: anno_obj
        """
        anno_obj = dict()

        img = cv2.imread(img_path)
        if len(img.shape) == 3:
            h, w, d = img.shape[:3]
            anno_obj['size'] = (w, h, d)
        else:
            h, w = img.shape[:2]
            anno_obj['size'] = (w, h, 1)

        anno_array = np.loadtxt(anno_path, dtype=np.str, delimiter=',')
        objects = list()
        if len(anno_array.shape) == 1:
            # Just one annotation object
            obj = self.create_anno(anno_array)
            if obj:
                objects.append(obj)
        else:
            for anno_line in anno_array:
                obj = self.create_anno(anno_line)
                if obj:
                    objects.append(obj)
        if len(objects) == 0:
            return dict()
        anno_obj['objects'] = objects
        return anno_obj

    def process(self) -> dict:
        src_dir = self.src_dir
        image_folder = self.image_folder
        label_folder = self.label_folder
        img_extension = self.img_extension
        anno_extension = self.anno_extension
        verbose = self.verbose
        logger = self.logger

        image_dir, label_dir = check_input_output_folder(src_dir, image_folder, label_folder, is_input=True)

        img_path_list = sorted(glob.glob(os.path.join(image_dir, '*' + img_extension)))
        anno_path_list = sorted(glob.glob(os.path.join(label_dir, '*' + anno_extension)))
        check_image_label(img_path_list, anno_path_list)

        anno_data = dict()
        for i, (img_path, anno_path) in enumerate(zip(img_path_list, anno_path_list), 1):
            if verbose:
                logger.info('parse {}'.format(anno_path))

            anno_obj = self.parse_anno(img_path, anno_path)
            if anno_obj:
                anno_data[img_path] = anno_obj
            else:
                self.logger.info(
                    '{} is a annotation object that only contains targets whose annotation type is ignored regions or others'.format(
                        img_path))

        return {'classmap': self.classmap, 'anno_data': anno_data}

    def save(self, input_data):
        super(VisDroneAnno, self).save(input_data)
        pass



# -*- coding: utf-8 -*-

"""
@date: 2021/4/3 下午3:16
@file: pytorch_imagefolder.py
@author: zj
@description: 
"""

import os
from torch.utils.data import DataLoader
from torchvision.datasets import ImageFolder

from . import registry
from .base_anno import BaseAnno
from ..util.logger import setup_logger


def raw_reader(path):
    with open(path, 'rb') as f:
        bin_data = f.read()
    return bin_data


@registry.ANNOS.register('imagefolder')
class PytorchImageFolder(BaseAnno):

    def __init__(self, cfg) -> None:
        self.name = cfg.IMAGEFOLDER.NAME

        if cfg.ANNO.PARSER == self.name:
            self.src_dir = cfg.INPUT.DIR
            self.image_folder = cfg.INPUT.IMAGE_FOLDER

        self.verbose = cfg.ANNO.VERBOSE
        self.logger = setup_logger(__name__)

    def process(self) -> dict:
        image_path = os.path.join(self.src_dir, self.image_folder)
        data_set = ImageFolder(image_path, loader=raw_reader)
        data_loader = DataLoader(data_set, num_workers=0)

        return {'dataloader': data_loader, 'classes': data_set.classes}

    def save(self, input_data: dict):
        super(PytorchImageFolder, self).save(input_data)
        pass



# -*- coding: utf-8 -*-

"""
@date: 2020/7/21 下午8:32
@file: cli.py
@author: zj
@description: 
"""

import sys
from ..config import cfg
from ..engine import Processor, default_argument_parser
from ..util.misc import get_version


def main():
    parser = default_argument_parser()
    args = parser.parse_args()

    if args.version:
        print('pnno: v{}'.format(get_version()))
        sys.exit(0)

    if not args.config_file:
        parser.print_usage()
        sys.exit(1)

    cfg.merge_from_file(args.config_file)
    cfg.freeze()

    processor = Processor(cfg)
    processor.process()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

"""
@date: 2020/7/13 下午4:03
@file: __init__.py.py
@author: zj
@description: 
"""

# This line will be programatically read/write by setup.py.
# Leave them at the bottom of this file and don't touch them.
__version__ = "0.3.4"



# -*- coding: utf-8 -*-

"""
@date: 2020/8/20 下午9:27
@file: show.py
@author: zj
@description: 
"""

import os
import cv2
import sys
import argparse
from ..util.misc import get_version


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('-f', "--img_file", default="", metavar="IMG_FILE", help="path to img file")
    parser.add_argument('-v', '--version', help='output version information', action="store_true")

    return parser


def show_image():
    parser = parse_args()
    args = parser.parse_args()

    if args.version:
        print('pnno: v{}'.format(get_version()))
        sys.exit(0)

    if not args.img_file:
        parser.print_usage()
        sys.exit(1)

    img_path = args.img_file
    if not os.path.exists(img_path):
        parser._print_message(f'{img_path}不存在\n')
        exit(0)
    if not os.path.isfile(img_path):
        parser._print_message(f'{img_path}不是文件\n')
        exit(0)

    suffix = os.path.splitext(os.path.basename(img_path))[1]
    if suffix.lower() not in ['.jpg', '.png', '.tif', '.bmp']:
        parser._print_message('当前支持jpg/png/tif/bmp格式图像的显示\n')
        exit(0)

    img_name = os.path.basename(img_path)
    img = cv2.imread(img_path)
    cv2.namedWindow(img_name, cv2.WINDOW_NORMAL)
    cv2.imshow(img_name, img)
    cv2.waitKey(0)


if __name__ == '__main__':
    show_image()



# -*- coding: utf-8 -*-

"""
@date: 2020/7/14 下午8:34
@file: processor.py
@author: zj
@description: 
"""

from ..anno import build_anno
from ..util.logger import setup_logger


class Processor(object):
    """
    The labeled data is processed to create training data with specified format
    """

    def __init__(self, cfg):
        self.parser = build_anno(cfg.ANNO.PARSER, cfg)
        self.creator = build_anno(cfg.ANNO.CREATOR, cfg)

        self.logger = setup_logger(__name__)
        self.verbose = cfg.ANNO.VERBOSE

    def process(self):
        verbose = self.verbose
        logger = self.logger

        if verbose:
            logger.info('Processing original data')
        output_data = self.parser.process()
        if verbose:
            logger.info('Save data in specified format')
        self.creator.save(output_data)
        if verbose:
            logger.info('Finish!!!')
